#include <pspkernel.h>
#include <pspdebug.h>
#include <psputils.h>
#include <pspsdk.h>
#include <pspctrl.h>
#include <stdlib.h>
#include <string.h>
#include "usb.h"
#include "sha1.h"

#define printf	pspDebugScreenPrintf

PSP_MODULE_INFO("PS3Jig", 0x1000, 1, 1);
PSP_MAIN_THREAD_ATTR(0); 

int done = 0;

int exit_callback (int arg1, int arg2, void *common)
{
	done = 1;
	return 0;
}

int CallbackThread (SceSize args, void *argp)
{
	int cbid;

	cbid = sceKernelCreateCallback("Exit Callback", exit_callback, NULL);
	sceKernelRegisterExitCallback(cbid);
	sceKernelSleepThreadCB();

	return 0;
}

int SetupCallbacks (void)
{
	int thid = 0;

	thid = sceKernelCreateThread("update_thread", CallbackThread,
				     0x11, 0xFA0, 0, 0);
	if (thid >= 0)
	{
		sceKernelStartThread(thid, 0, 0);
	}

	return thid;
}

int main (void)
{
    SetupCallbacks();
    pspDebugScreenInit();
    
    printf("PS3Jig\nBrandon Wilson\n\n");
    printf("I'm not a PSP developer, I don't presume to have any idea\n");
    printf("  what I'm doing; use at your own risk.\n\n");
    printf("Press O to attempt to quit (but you'll probably have to\n");
    printf("  pull the battery).\n\n");

	sceCtrlSetSamplingCycle(0);
	sceCtrlSetSamplingMode(PSP_CTRL_MODE_ANALOG);

	//Start our USB driver
	USB_Register();
	printf("USB driver registered.\n");
	USB_Start();
	printf("USB driver started.\n");

	//Main loop
	SceCtrlData pad;
	while (!done)
	{
		//Read keypad
		sceCtrlReadBufferPositive(&pad, 1); 
		if (pad.Buttons != 0)
		{
			if (pad.Buttons & PSP_CTRL_CIRCLE)
			{
				while (pad.Buttons & PSP_CTRL_CIRCLE) sceCtrlReadBufferPositive(&pad, 1); //debounce the key
				done = 1;
			}
    
		}
	}
	
	//Stop our USB driver
	USB_Stop();
	USB_Unregister();

	//Attempt to exit?
	sceKernelExitGame();
	return 0;
}
